#include "capd/capdlib.h"
#include <iostream>
using namespace std;
using namespace capd;
using namespace capd::matrixAlgorithms;

#include "localMapsFromData.h"
#include "covering.h"

bool validateTransitivityAndHyperbolicity(interval xi,IMap &F,int i)
{
	vector<localMap> f=getLocalMaps(xi,F,i);
	int m=f.size();
	
	IVector M(3);
	M[0]=interval(-0.1,0.1);
	M[1]=interval(-100,100);
	M[2]=interval(-0.4,0.4);
	vector<IVector> K(m+1);
	for(int j=0;j<m+1;j++) K[j]=M;
	
	for(int j=0;j<m;j++)
	{
		// We check the condition needed for transitivity and hyperbolicity of the invariant set in the union of the 
		// the sets K[j]. This condition is also needed for transitivity of the invariant set.
		if(checkHyperbolicity(f[j],K[j])==0)
		{
			cout << "Problem with uniform hyperbolicity condition needed for transitivity. Aborting program." << endl;
			return 0;
		}
		// This validates the covering and computes K[j+1] along the K[j+1][1] coordinate.
		if(isCovering2(f[j],K[j],K[j+1])==0) 
		{
			cout << "Problem with covering condition needed for transitivity. Aborting program." << endl;
			return 0;
		}
	}
	// We need to check that we have K[m-1]=>M. We know that K[m-1]=>K[m] so it is enough to check
	// that M[1] is a subset of K[m][1]:
	if(subsetInterior(M[1],K[m][1])==0)
	{
		cout << "Problem with final covering condition needed for transitivity. Aborting program." << endl;
		return 0;
	}
	return 1;
}

////////////////////
// Function: ValidateConnectingSequence()
//
// This routine validates coverings:
//   Ni0c => Ni1c => Ni2c => Ni3c => Ni4c => M    for i=1 and c=0,...,49
//   Ni0c => Ni1c => Ni2c => Ni3c => M    for i=2 and c=0,...,49
// for a given parameter xi and given i, which are passed to the function.
//
// It also validates cone conditions and the strong hyperbolicity condition. The latter ensures
// uniform hyperbolicity and transitivity of the invariant set.
bool ValidateConnectingSequence(interval xi,IMap &F,int i)
{
	// here we set up the sequence of covering maps
	vector<localMap> f=getLocalMaps(xi,F,i);

	int m=f.size();
	
	// These sequences of IVectors (cubes) will be the h-sets along the sequence of coverings.
	vector<IVector> N(m+1);
	
	// The coordinates for the h-sets in our program are (x,yu,ys):
	//    x - is strongly expanding,
	//    yu - is weakly expanding,
	//    ys - is contracting
	// This is the mother set: 
	IVector M(3);
	M[0]=0.1*interval(-1,1);  // x
	M[1]=interval(-2,2);  // yu
	M[2]=0.4*interval(-1,1);  // ys
	 
	// On the coordinates x,ys all our h-sets, when expressed in local coordinates, will be of the same size.
	// Below we validate the condition ineeded for uniform hyperbolicity and for the transitivity of the invariant set.
	for(int k=0;k<=m;k++) N[k]=M;

	// Now we check cone conditions.
	// This is the initial cone in the mother set M:
	IVector cone0(3);
	interval kappa=interval(2.0)/interval(100.0);
	cone0[0]=interval(1.0);
	cone0[1]=kappa*interval(-1,1);
	cone0[2]=kappa*interval(-1,1);
	
	// The cone is propagated along the sequence of h-sets.
	// Again, we validate the cone conditions on large sets. This ensures that the cone conditions will hold
	// for the sequences of h-sets involved in the coverings.
	IVector cone1 = propagateCone(N,cone0,f);
	// We validate that after we return the cones1 maps into the initial cone0 at the mother set:
	if(subsetInterior(cone1[1],cone0[1])==0) return 0; // if the function returns zero, our program will stop automatically.
	if(subsetInterior(cone1[2],cone0[2])==0) return 0;

	int J=50;
	interval Delta=2*kappa;
	
	for(int c=0;c<J;c++)
	{
		// here we set up our initial h-sets.
		// They are the same along the x,ys coordinates as the mother set M,
		// so here we only specify the coordinate yu. 
		// The entire interval spanned by us is
		//  [-h,(J-1)*h+h] = [-2*r,(J-1)*2*r+2*r] = [-2*r,49*2*0.02+2*0.02] = [-2*r,2] for i=1
		//  [-(J-1)*h-h,h] = [-(J-1)*2*r-2*r,2*r] = [-49*2*0.02-2*0.02,2*r] = [-2,2*r] for i=1
		if(i==1) N[0][1]= c*interval(Delta)+interval(Delta)*interval(-1,1);
		if(i==2) N[0][1]=-c*interval(Delta)+interval(Delta)*interval(-1,1);
		
		for(int k=0;k<m;k++)
		{	
			// here we validate the covering
			//   N[k] => N[k+1]
			// isCovering() also automatically computes the size of 
			// N[k+1][1], which is the yu coordinate.
			if(isCovering(f[k],N[k],N[k+1])==0)
			{
				cout << "Covering failed in intermediate step. Aborting program. " << endl;
				return 0;
			}
			
			// We chack that all our h-sets considered in the sequences of coverings are within the interval [-4,4]
			// along the yu coordinate. We need to do this since then we will know that our test of 
			// uniform hyperbolicity was correctly validated.
			if(!(subsetInterior(N[k+1][1],interval(-100,100))))
			{
				cout << N[k+1][1] << endl;
				cout << "The bounds needed for the uniform hyperbolicity were not computed on sufficiently large sets. Aborting program." << endl;
				return 0;
			}
		}
		// Here we chack that 
		//   N[m-1] => M
		// The topological alignment along coordinates x,ys is checked by the fact that we have validated
		//   N[m-1] => N[m]
		// since N[m] and M are the same on the coordinates x,ys. What remains is that we have entry along 
		// coordinate yu. For this it es enough that we know that N[m][1] is a subset of M[1].
		if(subsetInterior(N[m][1],M[1])==0)
		{
			cout << "Covering failed in final step. Aborting program. " << endl;
			cout << N[m][1] << endl;
			return 0;
		}
	}
	
	// If our program reaches this point, this means that all the conditions 
	// along the chosen sequences of coverings have been validated. Otherwise the routine would have returned zero.
	return 1;
}

int main(int argc, char* argv[])
{
  	try
    {	
        clock_t start, end;
        double time;
        start = clock();
		
		IMap f("par:mu,beta,xi;var:x,y,z;fun:y,mu+y^2+beta*x,xi*z+y;");
		f.setParameter("mu",-9.5);
		f.setParameter("beta",0.3);
		
		for(int i=0;i<115;i++)
		{
			interval xi=1.01+i*interval(1)/interval(1000)+interval(0,1)/interval(1000);
			if(validateTransitivityAndHyperbolicity(xi,f,1)==0) return 0;
			if(validateTransitivityAndHyperbolicity(xi,f,2)==0) return 0;
			if(ValidateConnectingSequence(xi,f,1)==0) return 0;
			if(ValidateConnectingSequence(xi,f,2)==0) return 0;
			cout << "For xi in " << xi << " the validation was successful. "<< endl;
		}
		
		end = clock();
        time = (double(end) - double(start)) / CLOCKS_PER_SEC;
		cout << "computation time: " << time << endl;	
    }
  	catch(exception& e)
    {
      cout << "\n\nException caught: " << e.what() << endl;
    }
    return 0;
}