#include "covering.h"

bool isCovering(const localMap &f,IVector N,IVector &M)
{
	// to ensure covering the set M has to be larger than f(N) along the 
	// entry coordinates. The parameter delta is used later on in the function 
	// to enlarge the set M.
	interval delta=power(interval(10),-10);
	
	// The Nl and Nr are the left and the right components of the exit set
	IVector Nl=N, Nr=N;
	Nl[0]=Nl[0].left();
	Nr[0]=Nr[0].right();
	
	IVector fNl=f(Nl), fNr=f(Nr), fN=f(N);
	
	// We validate that the images of the two components of the exit sets are mapped 
	// to the opposite sides, outside of M[0].
	if(!(abs(fNl[0])>abs(M[0]))) return 0;
	if(!(abs(fNr[0])>abs(M[0]))) return 0;
	if(!(fNr[0]*fNl[0]<0.0))  return 0;
	
	// We enlarge the set M[1] so that the entry condition is fulfilled along this coordinate
	M[1]=fN[1]+delta*interval(-1,1);
	
	// We validate the entry condition along the last coordinate.
	return subsetInterior(fN[2],M[2]);
}

bool isCovering2(const localMap &f,IVector N,IVector &M)
{
	interval delta=power(interval(10),-10);
	
	// The Nl and Nr are the left and the right components of the exit set.
	// The Nb and Nt are the bottom and the top components of the exit set.
	IVector Nl=N, Nr=N, Nb=N, Nt=N;
	Nl[0]=Nl[0].left();
	Nr[0]=Nr[0].right();
	Nb[1]=Nb[1].left();
	Nt[1]=Nt[1].right();
	
	IVector fNl=f(Nl), fNr=f(Nr), fN=f(N), fNt=f(Nt), fNb=f(Nb); 
	
	// We validate that the images of the left and the right exit sets are mapped 
	// to the opposite sides, outside of M[0].
	if(!(abs(fNl[0])>abs(M[0]))) return 0;
	if(!(abs(fNr[0])>abs(M[0]))) return 0;
	if(!(fNr[0]*fNl[0]<0.0))  return 0;
	
	// We ensure that the top component maps above M and the bottom component maps below M.
	// This is ensured by choosing M[1] appropriately small:
	interval Mb=fNb[1].right()+delta;
	interval Mt=fNt[1].left()-delta;
	
	if(!(Mt>Mb)) return 0;
	M[1]=intervalHull(Mb,Mt);
	
	// We validate the entry condition along the last coordinate.
	return subsetInterior(fN[2],M[2]);
}

IVector propagateCone(IVector N,IVector Cone,const localMap &f)
{
	// In Lemma 7 from the paper we have the parameters kappa_u^j, and kappa_s^j.
	// These need to be sufficiently large to ensure (19).  The parameter delta is used to
	// enlarge the bounds from the left hand side of (19). This results in the suitable choice
	// of the cone.
	interval delta=power(interval(10),-10);
	Cone = f[N]*Cone;
	
	Cone[1] = Cone[1]/Cone[0]; // this corresponds to the left hand side of (19)
	Cone[2] = Cone[2]/Cone[0]; // this corresponds to the left hand side of (19)
	Cone[1] = (Cone[1]+delta)*interval(-1,1); // we choose suitable kappa_u^j. Here Cone[1]=[-kappa_u^j,kappa_u^j]
	Cone[2] = (Cone[2]+delta)*interval(-1,1); // we choose suitable kappa_s^j. Here Cone[1]=[-kappa_s^j,kappa_s^j]
	
	Cone[0]=interval(1.0);
	return Cone;
}

IVector propagateCone(vector<IVector> N,IVector Cone,const vector<localMap> &f)
{
	int m=f.size();
	for(int i=0;i<m;i++) Cone=propagateCone(N[i],Cone,f[i]);
	return Cone;
}

bool checkHyperbolicity(const localMap &f,IVector N)
{
	IMatrix Q(3,3);
	Q[0][0]=1.0;
	Q[1][1]=1.0;
	Q[2][2]=-1.0;
	IMatrix D=transpose(f[N])*Q*f[N]-Q;
	
	// We validate that the matrix D is strictly positive definite by computing the determinants:
	// -) of the whole matrix:
	if(!(det(D)>0)) return 0.0;
	
	// -) of the top left 2 x 2 submatrix:
	if(!(D[0][0]*D[1][1]-D[1][0]*D[0][1]>0)) return 0.0;
	
	// -) of the top left coefficient:
	if(!(D[0][0]>0)) return 0.0;
	
	return 1;
}

bool checkHyperbolicity(const vector<localMap> &f,vector<IVector> N)
{
	int m=f.size();
	for(int i=0;i<m;i++)
	{
		if(checkHyperbolicity(f[i],N[i])==0) return 0;
	}
	return 1;
}