#ifndef localMapsFromData_h
#define localMapsFromData_h

#include "localMap.h"

///////////////////////
// Function: getLocalMaps()
//
// This routine returns a sequence of local maps associated with a sequence of coverings:
//
// Ni0c => Ni1c => Ni2c => Ni3c => Ni4c => M    for i=1 and c=0,...,49
// Ni0c => Ni1c => Ni2c => Ni3c => M    for i=2 and c=0,...,49
//
// The local coordinates for the local maps are defined by affine coordinate changes 
//   q+A*x
// where q is the midpoint for the local coordinates and A is a 3x3 matrix.
// Each set Nijc is associated with an approriate coordinate change; i.e. with an appropriate choice of q and A. 
//
// The choice of the matrix A=Aij for the local change of coordinates at a set Nijc 
// depends only on the choice of i and j. (It does not depend on c.) These matrices Aij
// are written out explicitly in the paper, and are also contained in the data files:
//    data/A1.txt    which contains matrices A10,...,A15
//    data/A2.txt    which contains matrices A20,...,A24
// The sequences of matrices are read by the routine 
//   vector<IMatrix> coordinates(int i);
// with i=1,2.
//
// IMPORTANT: In the function getLocalMaps() we make sure that we have the same linear change of coordinates:
//    A10=A20=A15=A24=AM
// This is important since the coordinates of the initial sets Ni0c must be the same as those of the mother set M. 
// We also must return to the initial coordinates of the set M after completing the sequence of coverings. 
//
// The midpoints q=qij(xi) used for the local coordinates also depend on the choice of i and j, but they also 
// depend on the choice of the parameter xi. In more detail: The coordinates X,Y of the points qij depend 
// only on i and j. The coordinate Z depends on i,j and xi.
// 
// The X,Y coordinates of the seqiences q10,...,q15 and q20,...,q24 are contained in the files:
//    data/points1.txt
//    data/points2.txt
// respectively. The Z coordinate is computed by a numerical procedure described in section 4.1 from the paper.
// This procedure requires the choice of Z at the point pM(xi) ath the beginning of a sequence. 
// This initial Z is computed by using the function pM(xi).
// The sequences of points are read by the routine 
//   vector<IVector> points(int i,interval xi);
// with i=1,2. (The computer assisted proof works for xi from the range [1.01,1.125].)
//
// IMPORTANT: In the function getLocalMaps() we make sure that we have the same points:
//    q10(xi)=q20(xi)=q15(xi)=q24(xi)=pM(xi).
// This is important since the coordinates of the initial sets Ni0c must be the same as those of the mother set M.
// We also must return to the initial coordinates of the set M after completing the sequence of coverings. 
//
// IMPORTANT: Let us emppasize that we are free to choose any pM(xi) and any matrix AM that we wish for the mothe set M.
// We are also completely free to choose any matrices Aij and any point qij(x) to define our local maps.
// As long as we start and finish in the same coordinate system as the mother set M and as long as we validate 
// the covering relations, cone conditions, and uniform hyperbolicity using the chosen local coordinates, the proof is valid.
// Good coordinate choices are not unique. The proof will work also with other choices (for instance small perturbations 
// of the ones chosen here). The computer assisted proof therefore consists of two parts:
//   1) Choice of some local coordinates, but making sure to start and finish at the mother set M.
//      (The choices here are in many ways arbitraty and dictated only by the nature of the system.
//      the h-sets are positioned more or les at the intersections of the stable/unstable manifolds,
//      but we are free to choose these sets as we wish. )
//   2) The computer assisted proof: once we have chosen coordinates and sets we need to validate 
//      the assumptions of our theorems. If the choices of coordinates in point 1) were poor,
//      then the computer assisted proof will not go through and the program will return an error.
//      If the choices were good enough, then the validation of the conditions will provide the proof.
//   
// Remark: If the user wishes to test this she/he could slightly modify the data from
//    data/points1.txt
//    data/points2.txt
//    data/A1.txt
//    data/A2.txt
// small alterations of these values should also lead to valid computer assisted proof.

vector<localMap> getLocalMaps(interval xi,IMap &f,int i);

///////////////////////
// Function: coordinates()
// 
// This reads the sequences of matrices:
//   A10,...,A15
// and
//   A20,...,A24
// from the files 
//    data/A1.txt
// and
//    data/A2.txt
// respectively.
vector<IMatrix> coordinates(int i);

//////////////////////////
// Function: points()
// 
// This reads the X,Y coordinates of points
//    q10(xi),...,q15(xi)
// and
//    q20(xi),...,q24(xi)
// from the files
//    data/points1.txt
// and
//    data/points2.txt
// respectively. After that it chooses the Z coordinates of these points depending on the choice of xi.
vector<IVector> points(int i,interval xi);

#endif
